
module EMD_tb();

	//////////////////PARAMETERS///////////////////////////
	
	parameter DATA_SIZE_BITS 		  = 16;	//Number of bits representing every sample in the input signal.
	parameter SIGNAL_LENGTH 		  = 2048;//Equivalent to the window size N 
	parameter PRECISION_BITS_CS 	  = 22;	//Added precision bits for cubic spline calculation.
	parameter PRECISION_BITS_MEMORY = 10;	//Precision bits used when storing (Maximum value : PRECISION_BITS_CS - 1).
	parameter CHECKING_PRECISION 	  = PRECISION_BITS_MEMORY;	//Used only for testing the performance when reducing the precision for the IMF_check (Can be ignored. Set to equal PRECISION_BITS_MEMORY)
	parameter NUMBER_OF_IMFS 		  = 11;	//Number of IMF the module should return. The module returns 0 if no more IMFs are available. 

	//////////////////REGISTERS/////////////////////////////
	
	reg 																		clk;
	reg 																		rst_n;
	reg											[DATA_SIZE_BITS-1:0] data_in;
	reg 																		data_in_available;

	//////////////////WIRES////////////////////////////////
	
	wire 																		data_out_available;
	wire signed 								[DATA_SIZE_BITS-1:0] data_out;
	wire signed [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] data_out_precision;
	wire																		EMD_finished;

	//////////////////MODULES///////////////////////////////
	
	EMD #(
			DATA_SIZE_BITS, 
			PRECISION_BITS_CS, 
			PRECISION_BITS_MEMORY, 
			CHECKING_PRECISION, 
			SIGNAL_LENGTH, 
			NUMBER_OF_IMFS
		  )
		 
			uut
		  (
		   clk, 
			rst_n, 
			data_in, 
			data_in_available, 
			data_out, 
			data_out_precision, 
			data_out_available, 
			EMD_finished
		  );
		  
	//////////////////INTEGERS//////////////////////////////		  

	integer file;
	integer result_file;
	integer scan_file;
	integer i;

	//////////////////Initializing the test/////////////////		
	
	initial begin
		file = $fopen("C:/Users/Lars/Documents/Master_attachments/Matlab/test_data.dat", "r");  		//Change the location to the folder containing "test_data.dat"
		result_file = $fopen("C:/Users/Lars/Documents/Master_attachments/Matlab/results.dat", "w");   //Change the location for where to store "results.dat"
		clk<=1;
		rst_n <= 1;
		data_in <= 0; 
		data_in_available<= 0;
		#1 rst_n <= 0;
		#1 rst_n <= 1;
	end
	
	//////////////////Reading the test data from file//////
	
	initial begin 
		@(negedge clk)
		data_in_available <= 1;
		for (i = 0; i<SIGNAL_LENGTH; i = i+1)begin
			scan_file = $fscanf(file, "%d\n", data_in);
			#10;
		end
		data_in_available <= 0;
	end
	
	//////////////////Clock generation/////////////////////
	
	always #5 clk = !clk;

	//////////////////Write results to file////////////////	
	
	always @(negedge clk)begin
		if(data_out_available)	
			$fwrite (result_file, "%d\n",data_out_precision);
	end
	
	//////////////////Stop////////////////////////////////
	
	initial begin
		#1500000 $stop; 
	end 

endmodule
