

module calculate_primes 

	//////////////////PARAMETERS///////////////////////////

#(
	parameter 
	DATA_SIZE_BITS = 16, 
	PRECISION_BITS_CS = 16,
	PRECISION_BITS_MEMORY = 8,
	DEPTH =512
)

	//////////////////INPUTS AND OUTPUTS////////////////////

( 
	input																						clk,
	input 																  				   rst_n,
	input signed 									 				[clogb2(DEPTH):0] A,
	input signed 								  				 [clogb2(DEPTH)+2:0] B,
	input signed 													[clogb2(DEPTH):0] C,
	input signed 				[DATA_SIZE_BITS + PRECISION_BITS_MEMORY+3:0] D,
	input 																	 				data_in_available,
	input																		 				first,
	input																						last,
	output reg 								 				[PRECISION_BITS_CS-1:0] C_prime,
	output reg 		  	 				[DATA_SIZE_BITS + PRECISION_BITS_CS:0] D_prime,
	output reg 																 				first_out,
	output reg																 				last_out,
	output reg																 				data_out_available
);

	//////////////////WIRES////////////////////////////////

	wire signed 							[clogb2(DEPTH)+PRECISION_BITS_CS:0] A_precision_cs;
	wire signed							 [clogb2(DEPTH)+PRECISION_BITS_CS+2:0] B_precision_cs;
	wire signed				 			 [clogb2(DEPTH)+2*PRECISION_BITS_CS:0] C_precision_cs;
	wire signed 		 		[DATA_SIZE_BITS + 2*PRECISION_BITS_CS + 3:0] D_precision_cs;
	wire signed 					   [DATA_SIZE_BITS + PRECISION_BITS_CS:0] debug_one;
	wire signed 				  [2*DATA_SIZE_BITS + 2*PRECISION_BITS_CS:0] debug_two;
	wire signed 						[DATA_SIZE_BITS + PRECISION_BITS_CS:0] debug_three;
	wire signed [DATA_SIZE_BITS + 2*PRECISION_BITS_CS + clogb2(DEPTH):0] A_D_prime;
	wire signed [DATA_SIZE_BITS + 2*PRECISION_BITS_CS + clogb2(DEPTH):0] D_m_A_D_prime;
	wire signed 					 [DATA_SIZE_BITS + 2*PRECISION_BITS_CS:0] D_prime_precision;
	
	//////////////////COMBINATORIAL////////////////////////
	
	//-----------------------------------------------------
	//			Applying precision bits
	//-----------------------------------------------------
	assign A_precision_cs										  [clogb2(DEPTH)+PRECISION_BITS_CS:PRECISION_BITS_CS] = A;
	assign A_precision_cs																				[PRECISION_BITS_CS-1:0] = 0;

	assign B_precision_cs										[clogb2(DEPTH)+PRECISION_BITS_CS+2:PRECISION_BITS_CS] = B;
	assign B_precision_cs																				[PRECISION_BITS_CS-1:0] = 0;


	assign C_precision_cs								  [clogb2(DEPTH) + 2*PRECISION_BITS_CS:2*PRECISION_BITS_CS] = C;
	assign C_precision_cs										  									 [2*PRECISION_BITS_CS-1:0] = 0;

	assign D_precision_cs [DATA_SIZE_BITS + 2*PRECISION_BITS_CS+3:2*PRECISION_BITS_CS-PRECISION_BITS_MEMORY] = D;
	assign D_precision_cs											 [(2*PRECISION_BITS_CS-PRECISION_BITS_MEMORY)-1:0] = 0;

	assign D_prime_precision 								[DATA_SIZE_BITS + 2*PRECISION_BITS_CS:PRECISION_BITS_CS] = D_prime;
	assign D_prime_precision																			[PRECISION_BITS_CS-1:0] = 0;

	assign A_D_prime 		= (A*$signed(D_prime))<< PRECISION_BITS_CS;
	assign D_m_A_D_prime = D_precision_cs - A_D_prime;

	//////////////////SEQUENTIAL///////////////////////////	
	
	//-----------------------------------------------------
	//			first_out, last_out, data_out_available
	//-----------------------------------------------------
	always @(posedge clk, negedge rst_n) begin
		if(!rst_n) begin
			last_out <= 0;
			first_out <= 0;
			data_out_available <= 0;
		end
		else begin
			last_out <= last;	
			first_out <= first;
			data_out_available <= data_in_available;
		end
	end

	//-----------------------------------------------------
	//			C_prime, D_prime
	//-----------------------------------------------------
	always @(posedge clk or negedge rst_n) begin
	if(!rst_n) begin
			C_prime <= 0;		
			D_prime <= 0;
		end
		else if(data_in_available) begin
			C_prime <= $signed(C_precision_cs)/$signed((B_precision_cs - A*C_prime));					
			D_prime <= D_precision_cs/$signed((B_precision_cs - A*C_prime)) - $signed(D_prime_precision)/$signed(B_precision_cs - A*C_prime)*A;
		end
	end


	//-----------------------------------------------------
	//			Function for calculating floor(log2(N-1))
	//-----------------------------------------------------
		function integer clogb2;
		input [31:0] depth;
			begin
				depth = depth-1;
				for(clogb2 = 0; depth>0; clogb2 = clogb2 +1)
					depth = depth >> 1;
			end
		endfunction

	endmodule
