
module collect_and_avg

	//////////////////PARAMETERS///////////////////////////
	 
#(
	parameter 
	DATA_SIZE_BITS = 16,
	DEPTH 			= 100
)

	//////////////////INPUTS AND OUTPUTS////////////////////
	 
(
	input 							clk,
	input [DATA_SIZE_BITS-1:0] data_max,
	input 							data_max_available,
	input [DATA_SIZE_BITS-1:0] data_min,
	input 							data_min_available,
	input 							first,
	input 							last,
	output[DATA_SIZE_BITS-1:0] out,
	output 							first_out,
	output 							last_out,
	output 							out_ready,
	input 							reset
);

	//////////////////REGISTERS/////////////////////////////

	reg 								rd_en_reg;

	//////////////////WIRES////////////////////////////////

	wire	 [clogb2(DEPTH)-1:0] nr_elem_in_q_fifo_one, nr_elem_in_q_fifo_two, nr_elem_in_q_fifo_last, nr_elem_in_q_fifo_first;
	wire	[DATA_SIZE_BITS-1:0] data_out_fifo_one;
	wire	[DATA_SIZE_BITS-1:0] data_out_fifo_two;
	wire 								data_out_fifo_first;
	wire 								data_out_fifo_last;
	wire 								rd_en;
	wire 								fifo_one_full;
	wire								fifo_one_empty;
	wire								fifo_two_full;
	wire								fifo_two_empty;
	wire								fifo_first_full;
	wire								fifo_first_empty;
	wire								fifo_last_empty;
	wire								fifo_last_full;
	wire 								data_out_fifo_one_updated;
	wire								data_out_fifo_two_updated;
	wire								data_out_fifo_last_updated;
	wire								data_out_fifo_first_updated;

	//////////////////MODULES////////////////////////////////

						 //Data_max 	
	FIFO #(
			 DATA_SIZE_BITS,
			 DEPTH
			)
			 
			 fifo_one
			 
			(
			 clk, 
			 rd_en, 
			 data_max_available, 
			 fifo_one_full, 
			 fifo_one_empty, 
			 nr_elem_in_q_fifo_one, 
			 data_max, 
			 data_out_fifo_one, 
			 data_out_fifo_one_updated, 
			 !reset
			);
			 
						 //Data_min 	
	FIFO #(
			 DATA_SIZE_BITS, 
			 DEPTH
			)
			 
			 fifo_two
			 
			(
			 clk,
			 rd_en,
			 data_min_available, 
			 fifo_two_full, 
			 fifo_two_empty, 
			 nr_elem_in_q_fifo_two, 
			 data_min, 
			 data_out_fifo_two, 
			 data_out_fifo_two_updated, 
			 !reset
			); 
			 
			 
						 //First	
	FIFO #(
			 1,
			 DEPTH
			)
			 
			 fifo_first
			
			(
			 clk, 
			 rd_en, 
			 data_min_available, 
			 fifo_first_full,
			 fifo_first_empty,
			 nr_elem_in_q_fifo_first,
			 first, data_out_fifo_first, 
			 data_out_fifo_first_updated, 
			 !reset
			); 
			 
						 //Last
	FIFO #(
			 1,
			 DEPTH
			)
			 
			 fifo_last
			 
			(
			 clk,
			 rd_en, 
			 data_min_available, 
			 fifo_last_full, 
			 fifo_last_empty, 
			 nr_elem_in_q_fifo_last,
			 last, data_out_fifo_last, 
			 data_out_fifo_last_updated, 
			 !reset
			); 

	average #(
				 DATA_SIZE_BITS
				)
				
				 avg
				 
				(
				 data_out_fifo_one, 
				 data_out_fifo_two, 
				 out
				);
				 
	//////////////////COMBINATORIAL////////////////////////
				 
	assign rd_en 	  = (!reset&(nr_elem_in_q_fifo_one>=1)&(nr_elem_in_q_fifo_two>=1)) ? 1 : 0;
	assign out_ready = rd_en_reg;
	assign first_out = data_out_fifo_first;
	assign last_out  = data_out_fifo_last & data_out_fifo_last_updated;  

	//////////////////SEQUENTIAL///////////////////////////

	always@(posedge clk or posedge reset) begin
	if(reset)
		rd_en_reg <= 0;
	else
		rd_en_reg <= rd_en;
	end

	//Function for calculating number of bits needed to cover DEPTH.

		function integer clogb2;
		input [31:0] depth;
			begin
				depth = depth-1;
				for(clogb2 = 0; depth>0; clogb2 = clogb2 +1)
					depth = depth >> 1;
			end
		endfunction
endmodule
