
module feeder_cubic_spline_start 

	//////////////////SEQUENTIAL///////////////////////////

#(
	parameter
	DATA_SIZE_BITS 		 = 16, 
	PRECISION_BITS_MEMORY = 8, 
	DEPTH 					 = 512
)

	//////////////////INPUTS AND OUTPUTS////////////////////

(
	input 																  clk,
	input 																  rst_n,
	input																	  point_is_extreme,
	input																	  data_in_available,
	input 																  first,
	input 	  															  last,
	input 	  [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] extremes,
	output reg 															  update,
	output 	 															  first_out,
	output reg 															  last_out,
	output reg [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] y_m,
	output reg [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] y,
	output reg [DATA_SIZE_BITS + PRECISION_BITS_MEMORY-1:0] y_p,
	output reg 									[clogb2(DEPTH)-1:0] h,
	output reg 									[clogb2(DEPTH)-1:0] h_m,
	output reg 									[clogb2(DEPTH)-1:0] distance_between,
	output reg 															  distance_between_available
);

	//////////////////REGISTERS/////////////////////////////

	reg 											[clogb2(DEPTH)-1:0] counter;
	reg 																	  ready;
	reg 															  [1:0] counter_ready;
	reg 																	  first_out_controller;

	//////////////////COMBINATORIAL////////////////////////
	
	assign first_out = update & !first_out_controller;

	//////////////////SEQUENTIAL///////////////////////////
	
	always @(posedge clk, negedge rst_n) begin
		if(!rst_n)
			last_out <= 0;
		else
			last_out <= last;	
	end

	always @(posedge clk, negedge rst_n) begin
		if(!rst_n)
			update <= 0;
		else
			update <= point_is_extreme & ready;
	end


	always@(posedge clk or negedge rst_n) begin

		if (!rst_n) begin
			h				<= 0;
			y_p			<= 0;
			y_m 			<= 0;
			y 				<= 0;
			h_m 			<= 0;
			distance_between <= 0;

		end
		else if(point_is_extreme) begin
			y_p <= extremes;
			y 	 <= y_p;
			y_m <= y;
			h 	 <= counter +1; 
			h_m <= h;
			distance_between <= counter +1;
		end
	end

	always @(posedge clk, negedge rst_n) begin
		if(!rst_n)
			distance_between_available <= 0;
		else
			distance_between_available <= point_is_extreme & !first;	
	end


	always@(posedge clk or negedge rst_n)begin
		if(!rst_n)
			ready <= 0;
		else if((counter_ready == 1)&& point_is_extreme)
			ready <= 1;		
	end 

	always@(posedge clk or negedge rst_n)begin
		if(!rst_n)
			counter <= 0;
		else if(data_in_available)begin
			if(point_is_extreme)
				counter <= 0;
			else
				counter <= counter +1;
		end
	end 


	always@(posedge clk or negedge rst_n)begin
		if(!rst_n)
			counter_ready <= 0;
		else if((counter_ready < 3) & point_is_extreme)
			counter_ready <= counter_ready +1;
	end 

	always@(posedge clk or negedge rst_n)begin
		if(!rst_n)
			first_out_controller <= 0;
		else if(update)
			first_out_controller <= 1;
	end 


	//Function for calculating number of bits needed to cover DEPTH.

		function integer clogb2;
		input [31:0] depth;
			begin
				depth = depth-1;
				for(clogb2 = 0; depth>0; clogb2 = clogb2 +1)
					depth = depth >> 1;
			end
		endfunction
endmodule
