function SNR = plot_and_compare(signal, samples, precision_bits_memory)
    %% Floating point precision
    figure('Name','IMFs Matlab, IMFs ModelSim, difference between them','NumberTitle','off');
    [IMFs_lars, r_lars, u_lars, l_lars, sift_counter] = emd_lars(signal);
    dim = size(IMFs_lars);
    
    for i = 1:dim(1)
        subplot(dim(1)+1,3,((i-1)*3)+1)
        plot(IMFs_lars(i,:))
        title(['IMF ', num2str(i), ', floating point precision'])
        xlim([0 samples])
    end
    
    subplot(dim(1)+1,3,(dim(1)*3)+1)
    plot(r_lars)
    title('Residue, floating point precision')
    xlim([0 samples])

    %% ModelSim: Fixed point precision
    [IMFs_quartus , nr_of_IMFs] = translate_file_from_quartus('results.dat',samples);
    IMFs_quartus = IMFs_quartus/(2^precision_bits_memory);
    
    for i = 1:dim(1)+1
        subplot(dim(1)+1,3,((i-1)*3)+2)
        plot(IMFs_quartus(i,:))
        if i == dim(1)+1
            title('Residue, fixed point precision') 
        else
            title(['IMF ', num2str(i), ', fixed point precision'])
        end
        xlim([0 samples])
    end

    for i = 1:dim(1)
        subplot(dim(1)+1,3,((i-1)*3)+3)
        diff(i,:) = IMFs_lars(i,:) - IMFs_quartus(i,:);
        plot(diff(i,:))
        title(['IMF ', num2str(i),' difference'])
        xlim([0 samples])
    end
    
    subplot(dim(1)+1,3,(((dim(1)+1)-1)*3)+3)
    plot(r_lars - IMFs_quartus((dim(1)+1),:))
    title(['Residue difference'])
    xlim([0 samples])

    for i = 1:dim(1)
        SNR(i) = sum(IMFs_lars(i,:).^2/sum(diff(i,:).^2));
    end
    
    h_avg_max = zeros(1,dim(1));
    h_avg_min = zeros(1,dim(1));
    x_max = zeros(1,dim(1));
    x_min = zeros(1,dim(1));
    
    for p = 1:dim(1)  
        tmp_signal = IMFs_lars(p,:);
        [yMax, xMax] = findpeaks(flip(tmp_signal));
        xMax = flip([samples , samples - (xMax-1), 1]);

        [yMin, xMin] = findpeaks(flip(-tmp_signal));
        xMin = flip([samples , samples - (xMin-1), 1]);
        h_max = zeros(1,length(xMax)-1);
        h_min = zeros(1,length(xMin)-1);
        
        for y = 1:length(xMax)-1
            h_max(y) = xMax(y+1) - xMax(y);
        end
        for t = 1:length(xMin)-1
            h_min(t) = xMin(t+1) - xMin(t);
        end

        h_avg_max(p) = sum(h_max)/length(h_max);
        h_avg_min(p) = sum(h_min)/length(h_min);
    end
    
    k = 1:dim(1);
    
    figure('Name','average distance and number of sifts for each IMF','NumberTitle','off');
    subplot(121)
    plot(k, h_avg_max)
    title('Average distance')
    xlabel('k')
    ylabel('Samples')
    legend('h_{avg}[k]')
    xlim([1, dim(1)]);
    grid on
    subplot(122)
    plot(k, sift_counter)
    title('Number of sift iterations')
    xlabel('k')
    ylabel('Number of sift iterations')
    legend('m[k]')
    xlim([1, dim(1)]);
    grid on
    
end


